;+
; NAME:
;
; counts2countrate
;
; PURPOSE:
;
; To convert from counts to counts per second
;
; CATEGORY:
;
; data handling
;
; CALLING SEQUENCE:
;
; counts2countrate,index,data
; 
; INPUTS:
;
; The index and data structures returned from rd_sumer
;
; OPTIONAL INPUTS:
;
; None.
;       
; KEYWORD PARAMETERS:
;
; None.
;
; OUTPUTS:
;
; The input data structure is overwritten.
;
; OPTIONAL OUTPUTS:
;
; None.
;
; COMMON BLOCKS:
;
; None.
;
; SIDE EFFECTS:
;
; The history variable is updated with 'counts2countrate'.
;
; RESTRICTIONS:
;
; The exposure time must be greater than zero. If, for some odd reason
; it isn't, nothing is done.
;
; PROCEDURE:
;
; Uses sgt_exptime to get exposure times.
;
; EXAMPLE:
;
; IDL> filename = '$SUMER_DATA/sum_960513_085055.fits'
; IDL> rd_sumer,index,data
; IDL> counts2countrate,index,data
;
; MODIFICATION HISTORY:
;
;   HPW 13-JAN-1997:  Orginal version.
;   
;   HPW 13-Jun-1997:  Added check on history.
;
;-

pro counts2countrate,index,data

n_cols = n_tags(data)

for n=0,n_cols-1 do begin

  exptime = sgt_exptime(index.(n))
  n_exp   = n_elements(exptime)

  for m=0,n_exp-1 do begin
    history = index.(n).spectrum(m).history
    check   = strpos(history,'counts2countrate')
    if (exptime(m) gt 0 and check eq -1) then begin
      ;; divide by exposure time
      data.(n)(*,*,m) = data.(n)(*,*,m)/exptime(m)

      ;; add note to history variable
      history = index.(n).spectrum(m).history
      if history eq '' then sep = '' else sep = ','
      index.(n).spectrum(m).history = history + sep + 'counts2countrate'
    endif else begin
      if exptime(m) le 0 then $
          message,'Pathological exposure time: '+trim(exptime(m)),/INFO
      if check ne -1 then $
          message,'Data has already been processed :'+history,/INFO
    endelse
  endfor

endfor

return
end
