
;+
; NAME:
;
; sgt_bin
;
; PURPOSE:
;
; To retrieve the binning parameters from a SOHO/SUMER observation.
;
; CATEGORY:
;
; Data handling. 
;
; CALLING SEQUENCE:
;
; bin = sgt_bin(index)
; 
; INPUTS:
;
; The index structure returned from rd_sumer.
;
; OPTIONAL INPUTS:
;
; None.
;       
; KEYWORD PARAMETERS:
;
; None.
;
; OUTPUTS:
;
; The wavelength and spatial binning performed on the data.
;
; OPTIONAL OUTPUTS:
;
; None.
;
; COMMON BLOCKS:
;
; None.
;
; SIDE EFFECTS:
;
; If the BINX and BINY tag names can not be found in the index
; structure passed to the routine, an error occurs.
;
; RESTRICTIONS:
;
; Only the index structure may be passed as input. It may turn out to
; be useful to pass the filename as input and this option may be added
; in the future.
;
; PROCEDURE:
;
; The routine returns the values of index.gen.binx and index.gen.biny
; from the index structure. The values of binx and biny have been
; taken directly from the primary FITS header.
;
; EXAMPLE:
;
; IDl> filename = '$SUMER_DATA/sum_960513_085055.fits'
; IDL> rd_sumer,index,data
; IDL> bin = sgt_bin(index)
; IDL> print,bin
;           1           1
; which indicates that no binning has been performed on the data.
;
; MODIFICATION HISTORY:
;
;   HPW 24-JUL-1996:
;   HPW 07-AUG-1996: Renamed sgt_bin.
;
;-

function sgt_bin,input

;; return to main level on error
on_error,1

case datatype(input,1) of

  ;; assume index structure has been passed
  'Structure': begin

    x = sgt_tagname(input,'BINX',status=status_x)
    y = sgt_tagname(input,'BINY',status=status_y)

    if (status_x and status_y) then begin
      bin = [x,y]
    endif else begin
      message,'Pathological index structure.'
    endelse
  end

  else: message,'Currently, only the index stucture may be passed as input.'
      
endcase

return,bin
end
