
;+
; NAME:
;
; sgt_tagname
;
; PURPOSE:
;
; To retrieve the value of structure tag name.
;
; CATEGORY:
;
; Data handling. 
;
; CALLING SEQUENCE:
;
; tag_value = sgt_tagname(structure,tag_name,status=status)
; 
; INPUTS:
;
; A structure and tag name.
;
; OPTIONAL INPUTS:
;
; None.
;       
; KEYWORD PARAMETERS:
;
; None.
;
; OUTPUTS:
;
; The value of structure tag name.
;
; OPTIONAL OUTPUTS:
;
; Status: Status is 0 (FALSE) if the input tag name is not found and 1
;         (TRUE) if it is. If status is false the routine returns a
;         null string ('') as output.
;
; COMMON BLOCKS:
;
; None.
;
; SIDE EFFECTS:
;
; None?
;
; RESTRICTIONS:
;
; Input variable must be a structure. Only the first instance of a tag
; name is returned. Additional sub-structures will not be searched
; after the first instance of the specified tag name is found.
;
; PROCEDURE:
;
; The routine recursively desends the stucture searching for the
; specified tag name.
;
; EXAMPLE:
;
; IDl> filename = '$SUMER_DATA/sum_960513_085055.fits'
; IDL> rd_sumer,index,/nodata
; IDL> date_obs = sgt_tagname(index,'date_obs',status=status)
; IDL> print,date_obs
;  1996-05-13T08:50:55.030
; IDL> print,status
;  1
;
; MODIFICATION HISTORY:
;
;   HPW 24-JUL-1996:
;   HPW 07-AUG-1996: Renamed sgt_tagname.
;
;-

function sgt_tagname,index,input_name,status=status

;; return to main level on error
on_error,1

TRUE   = 1
FALSE  = 0
status = FALSE
var    = ''

case datatype(index,1) of 

  'Structure': begin

    ;; match tag names to input string
    name_in = strupcase(input_name)
    names   = tag_names(index)
    match   = where(names eq name_in,count)
    
    if (count eq 1) then begin 
      var    = index.(match(0))
      status = TRUE
    endif else begin
      n_names  = n_elements(names)  
      continue = TRUE
      i_struct = 0
      while (continue) do begin
        if (datatype(index.(i_struct),1) eq 'Structure') then begin
          var = sgt_tagname(index.(i_struct),name_in,status=status)
          if (status) then continue = FALSE else continue = TRUE
        endif
        i_struct = i_struct + 1
        if (i_struct eq n_names) then continue = FALSE
      endwhile
    endelse
  end

  else: message,"Input variable is not a structure."

endcase
    
return,var
end

